"""``PeptideBuilder`` package for creating peptide models in PDB format based on geometrical parameters
Written by Matthew Z Tien.
Python modules
----------------
The package consists of the following Python modules:
* PeptideBuilder
* Geometry
"""
__version__ = '1.0.3'
from .PeptideBuilder import *
from .Geometry import *
